#include <ArduinoBLE.h>
/*
  This sketch demostrates the sending updated latest RTC information with each readValue()
  It acts as the pheripheral, thus it will advertise the service and characteristics


RTC test. Compiled on: 12:32:46
ready to go
Connected event, central: 62:b0:49:64:a8:95
It is now 12:03:25.95 11/14/22 Day of week: 1 = Monday
Sending string: 12:03:25    <-------------------- dummy value set on connect

String read call_Back is called!
It is now 12:03:54.03 11/14/22 Day of week: 1 = Monday
Sending string: 12:03:54    <-------------------- After waiting some time with first read the value is updated and I see on nrfcONNECT I see that : 31-32-3A-30-33-3A-35-34-0A

String read call_Back is called!
It is now 12:04:30.12 11/14/22 Day of week: 1 = Monday
Sending string: 12:04:30   <--------------------  After waiting some time with second read the value is updated and I see on nrfcONNECT I see that : 31-32-3A-30-34-3A-34-30-0A
*/

#include "RTC.h" // Include RTC library included with the Arduino_Apollo3 core

///////////////////////////////////////////////////////
// BLE defines
///////////////////////////////////////////////////////
const char BLE_PERIPHERAL_NAME[] = "RTC BLE";

#define CHARACTERISTIC_W_STR_UUID "9e400005-b5a3-f393-e0a9-e50e24dcca9e"

// create the service
BLEService Service("9e400001-b5a3-f393-e0a9-e50e24dcca9e");

// create characteristic
BLEStringCharacteristic s_w_Characteristic(CHARACTERISTIC_W_STR_UUID, BLERead | BLEWrite,100);

///////////////////////////////////////////////////////
// NO NEED FOR CHANGES BEYOND THIS POINT
///////////////////////////////////////////////////////

/**
 * called when ASCII / STRING reading is done by remote. so we write
 */
void StringReadCallBack(BLEDevice *central, BLECharacteristic c) {
    Serial.println("String read call_Back is called!");
    Send_RTC_data();
}

/**
 * create an updated RTC string and send
 */
void Send_RTC_data()
{
  String str;
  
  rtc.getTime();
  //
  // printf might not work for you
  //
  Serial.printf("It is now ");
  Serial.printf("%d:", rtc.hour);
  Serial.printf("%02d:", rtc.minute);
  Serial.printf("%02d.", rtc.seconds);
  Serial.printf("%02d", rtc.hundredths);

  Serial.printf(" %02d/", rtc.month);
  Serial.printf("%02d/", rtc.dayOfMonth);
  Serial.printf("%02d", rtc.year);

  Serial.printf(" Day of week: %d =", rtc.weekday);
  Serial.printf(" %s", rtc.textWeekday);

  char h[5], m[5],s[5];
  
  // workaround as %02d did not work for me with sprintf()
  // known issue with Artemis V2.x library
  uint32_t rv = rtc.hour;
  if ( rv < 10) sprintf(h,"0%d:", rtc.hour);  // add zero
  else sprintf(h,"%d:", rtc.hour);

  rv = rtc.minute;
  if ( rv < 10) sprintf(m,"0%d:", rtc.minute);  // add zero
  else sprintf(m,"%d:", rtc.minute);

  rv = rtc.seconds;
  if ( rv < 10) sprintf(h,"0%d\n", rtc.seconds);  // add zero
  else sprintf(s,"%d\n", rtc.seconds);
  
  str = String(h) + String(m) + String(s);

  Serial.print("\nSending string: ");
  Serial.print(str);
  
  s_w_Characteristic.writeValue(str); 
}

void setup() {
   Serial.begin(115200);
   delay(1000);
   Serial.print("\nRTC test. Compiled on: ");
   Serial.println(__TIME__);

  // Easily set RTC using the system __DATE__ and __TIME__ macros from compiler
  rtc.setToCompilerTime();

  ///////////////////////////////////////////////////
  // BLE SETUP START
  ///////////////////////////////////////////////////
  // begin initialization
  if (!BLE.begin()) {
    Serial.println(F("starting BLE failed!\r"));
    while (1);
  }

  // set the local name peripheral advertises
  BLE.setLocalName(BLE_PERIPHERAL_NAME);

  // set the UUID for the service this peripheral advertises:
  BLE.setAdvertisedService(Service);

  // add the characteristics to the service
  Service.addCharacteristic(s_w_Characteristic); // add String for us to write characteristic

  // add the service
  BLE.addService(Service);

  // set connection handlers
  BLE.setEventHandler(BLEConnected, blePeripheralConnectHandler);
  BLE.setEventHandler(BLEDisconnected, blePeripheralDisconnectHandler);

  // start advertising
  BLE.advertise();

  // called when read is performed by remote (and thus we write)
  s_w_Characteristic.setEventHandler(BLERead, (BLECharacteristicEventHandler) StringReadCallBack);

  ///////////////////////////////////////////////////
  // BLE SETUP END
  ///////////////////////////////////////////////////
  
  Serial.println(F("ready to go"));
}

void loop() {

  // wait for connection
  while (! BLE.connected() ){
    BLE.poll();
    delay(1000);     // don't hammer
  }

  BLE.poll();
  delay(500);        // don't hammer
}

/**
 * called when central connects
 */
void blePeripheralConnectHandler(BLEDevice central) {
  // central connected event handler
  Serial.print("Connected event, central: ");
  Serial.println(central.address());
  Send_RTC_data();    // set dummy data for correct _valueLength
}

/**
 * called when central disconnects
 */
void blePeripheralDisconnectHandler(BLEDevice central) {
  // central disconnected event handler
  Serial.print("Disconnected event, central: ");
  Serial.println(central.address());
}
